@@ -32,6 +32,8 @@
 #include "checksum.h"
 #include "array.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &mares_puck_device_vtable)
+
 #define NEMOWIDE    1
 #define NEMOAIR     4
 #define PUCK        7
@@ -48,7 +50,7 @@ static dc_status_t mares_puck_device_dump (dc_device_t *abstract, dc_buffer_t *b
 static dc_status_t mares_puck_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t mares_puck_device_close (dc_device_t *abstract);
 
-static const device_backend_t mares_puck_device_backend = {
+static const dc_device_vtable_t mares_puck_device_vtable = {
 	DC_FAMILY_MARES_PUCK,
 	mares_puck_device_set_fingerprint, /* set_fingerprint */
 	mares_common_device_read, /* read */
@@ -82,15 +84,6 @@ static const mares_common_layout_t mares_nemowide_layout = {
 	0x4000  /* rb_freedives_end */
 };
 
-static int
-device_is_mares_puck (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &mares_puck_device_backend;
-}
-
 
 dc_status_t
 mares_puck_device_open (dc_device_t **out, dc_context_t *context, const char *name)
@@ -106,7 +99,7 @@ mares_puck_device_open (dc_device_t **out, dc_context_t *context, const char *na
 	}
 
 	// Initialize the base class.
-	mares_common_device_init (&device->base, context, &mares_puck_device_backend);
+	mares_common_device_init (&device->base, context, &mares_puck_device_vtable);
 
 	// Set the default values.
 	device->layout = NULL;
@@ -273,7 +266,7 @@ mares_puck_extract_dives (dc_device_t *abstract, const unsigned char data[], uns
 {
 	mares_puck_device_t *device = (mares_puck_device_t*) abstract;
 
-	if (abstract && !device_is_mares_puck (abstract))
+	if (abstract && !ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (size < PACKETSIZE)
