@@ -108,16 +108,27 @@ static const oceanic_common_version_t oceanic_default_version[] = {
 
 static const oceanic_common_version_t tusa_zenair_version[] = {
 	{"TUZENAIR \0\0 512K"},
+	{"AMPHOSSW \0\0 512K"},
 };
 
 static const oceanic_common_version_t oceanic_oc1_version[] = {
 	{"OCWATCH R\0\0 1024"},
-	{"OCEANVT4 \0\0 1024"},
 	{"OC1WATCH \0\0 1024"},
-	{"OCEATOM3 \0\0 1024"},
 	{"OCSWATCH \0\0 1024"},
-	{"OCEAVT41 \0\0 1024"},
+};
+
+static const oceanic_common_version_t oceanic_oci_version[] = {
+	{"OCEANOCI \0\0 1024"},
+};
+
+static const oceanic_common_version_t oceanic_atom3_version[] = {
+	{"OCEATOM3 \0\0 1024"},
 	{"ATOM31  \0\0  1024"},
+};
+
+static const oceanic_common_version_t oceanic_vt4_version[] = {
+	{"OCEANVT4 \0\0 1024"},
+	{"OCEAVT41 \0\0 1024"},
 	{"AERISAIR \0\0 1024"},
 };
 
@@ -233,6 +244,45 @@ static const oceanic_common_layout_t oceanic_oc1_layout = {
 	1 /* pt_mode_logbook */
 };
 
+static const oceanic_common_layout_t oceanic_oci_layout = {
+	0x20000, /* memsize */
+	0x0000, /* cf_devinfo */
+	0x0040, /* cf_pointers */
+	0x10C0, /* rb_logbook_begin */
+	0x1400, /* rb_logbook_end */
+	8, /* rb_logbook_entry_size */
+	0x1400, /* rb_profile_begin */
+	0x1FE00, /* rb_profile_end */
+	0, /* pt_mode_global */
+	1 /* pt_mode_logbook */
+};
+
+static const oceanic_common_layout_t oceanic_atom3_layout = {
+	0x20000, /* memsize */
+	0x0000, /* cf_devinfo */
+	0x0040, /* cf_pointers */
+	0x0400, /* rb_logbook_begin */
+	0x0A40, /* rb_logbook_end */
+	8, /* rb_logbook_entry_size */
+	0x0A40, /* rb_profile_begin */
+	0x1FE00, /* rb_profile_end */
+	0, /* pt_mode_global */
+	1 /* pt_mode_logbook */
+};
+
+static const oceanic_common_layout_t oceanic_vt4_layout = {
+	0x20000, /* memsize */
+	0x0000, /* cf_devinfo */
+	0x0040, /* cf_pointers */
+	0x0420, /* rb_logbook_begin */
+	0x0A40, /* rb_logbook_end */
+	8, /* rb_logbook_entry_size */
+	0x0A40, /* rb_profile_begin */
+	0x1FE00, /* rb_profile_end */
+	0, /* pt_mode_global */
+	1 /* pt_mode_logbook */
+};
+
 static const oceanic_common_layout_t oceanic_veo1_layout = {
 	0x0400, /* memsize */
 	0x0000, /* cf_devinfo */
@@ -433,6 +483,12 @@ oceanic_atom2_device_open (dc_device_t **out, dc_context_t *context, const char
 		device->base.layout = &tusa_zenair_layout;
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_oc1_version)) {
 		device->base.layout = &oceanic_oc1_layout;
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_oci_version)) {
+		device->base.layout = &oceanic_oci_layout;
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_atom3_version)) {
+		device->base.layout = &oceanic_atom3_layout;
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_vt4_version)) {
+		device->base.layout = &oceanic_vt4_layout;
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_veo1_version)) {
 		device->base.layout = &oceanic_veo1_layout;
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_reactpro_version)) {
