@@ -31,6 +31,8 @@
 #include "ringbuffer.h"
 #include "checksum.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &oceanic_veo250_device_vtable)
+
 #define MAXRETRIES 2
 #define MULTIPAGE  4
 
@@ -51,7 +53,7 @@ typedef struct oceanic_veo250_device_t {
 static dc_status_t oceanic_veo250_device_read (dc_device_t *abstract, unsigned int address, unsigned char data[], unsigned int size);
 static dc_status_t oceanic_veo250_device_close (dc_device_t *abstract);
 
-static const device_backend_t oceanic_veo250_device_backend = {
+static const dc_device_vtable_t oceanic_veo250_device_vtable = {
 	DC_FAMILY_OCEANIC_VEO250,
 	oceanic_common_device_set_fingerprint, /* set_fingerprint */
 	oceanic_veo250_device_read, /* read */
@@ -85,16 +87,6 @@ static const oceanic_common_layout_t oceanic_veo250_layout = {
 };
 
 
-static int
-device_is_oceanic_veo250 (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &oceanic_veo250_device_backend;
-}
-
-
 static dc_status_t
 oceanic_veo250_send (oceanic_veo250_device_t *device, const unsigned char command[], unsigned int csize)
 {
@@ -240,7 +232,7 @@ oceanic_veo250_device_open (dc_device_t **out, dc_context_t *context, const char
 	}
 
 	// Initialize the base class.
-	oceanic_common_device_init (&device->base, context, &oceanic_veo250_device_backend);
+	oceanic_common_device_init (&device->base, context, &oceanic_veo250_device_vtable);
 
 	// Override the base class values.
 	device->base.layout = &oceanic_veo250_layout;
@@ -322,9 +314,6 @@ oceanic_veo250_device_close (dc_device_t *abstract)
 {
 	oceanic_veo250_device_t *device = (oceanic_veo250_device_t*) abstract;
 
-	if (! device_is_oceanic_veo250 (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Switch the device back to surface mode.
 	oceanic_veo250_quit (device);
 
@@ -346,7 +335,7 @@ oceanic_veo250_device_keepalive (dc_device_t *abstract)
 {
 	oceanic_veo250_device_t *device = (oceanic_veo250_device_t*) abstract;
 
-	if (! device_is_oceanic_veo250 (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	unsigned char answer[2] = {0};
@@ -373,7 +362,7 @@ oceanic_veo250_device_version (dc_device_t *abstract, unsigned char data[], unsi
 {
 	oceanic_veo250_device_t *device = (oceanic_veo250_device_t*) abstract;
 
-	if (! device_is_oceanic_veo250 (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (size < PAGESIZE)
@@ -404,9 +393,6 @@ oceanic_veo250_device_read (dc_device_t *abstract, unsigned int address, unsigne
 {
 	oceanic_veo250_device_t *device = (oceanic_veo250_device_t*) abstract;
 
-	if (! device_is_oceanic_veo250 (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	if ((address % PAGESIZE != 0) ||
 		(size    % PAGESIZE != 0))
 		return DC_STATUS_INVALIDARGS;
