@@ -29,6 +29,8 @@
 #include "parser-private.h"
 #include "array.h"
 
+#define ISINSTANCE(parser) dc_parser_isinstance((parser), &oceanic_vtpro_parser_vtable)
+
 typedef struct oceanic_vtpro_parser_t oceanic_vtpro_parser_t;
 
 struct oceanic_vtpro_parser_t {
@@ -45,7 +47,7 @@ static dc_status_t oceanic_vtpro_parser_get_field (dc_parser_t *abstract, dc_fie
 static dc_status_t oceanic_vtpro_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
 static dc_status_t oceanic_vtpro_parser_destroy (dc_parser_t *abstract);
 
-static const parser_backend_t oceanic_vtpro_parser_backend = {
+static const dc_parser_vtable_t oceanic_vtpro_parser_vtable = {
 	DC_FAMILY_OCEANIC_VTPRO,
 	oceanic_vtpro_parser_set_data, /* set_data */
 	oceanic_vtpro_parser_get_datetime, /* datetime */
@@ -55,16 +57,6 @@ static const parser_backend_t oceanic_vtpro_parser_backend = {
 };
 
 
-static int
-parser_is_oceanic_vtpro (dc_parser_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &oceanic_vtpro_parser_backend;
-}
-
-
 dc_status_t
 oceanic_vtpro_parser_create (dc_parser_t **out, dc_context_t *context)
 {
@@ -79,7 +71,7 @@ oceanic_vtpro_parser_create (dc_parser_t **out, dc_context_t *context)
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, context, &oceanic_vtpro_parser_backend);
+	parser_init (&parser->base, context, &oceanic_vtpro_parser_vtable);
 
 	// Set the default values.
 	parser->cached = 0;
@@ -95,9 +87,6 @@ oceanic_vtpro_parser_create (dc_parser_t **out, dc_context_t *context)
 static dc_status_t
 oceanic_vtpro_parser_destroy (dc_parser_t *abstract)
 {
-	if (! parser_is_oceanic_vtpro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Free memory.
 	free (abstract);
 
@@ -110,9 +99,6 @@ oceanic_vtpro_parser_set_data (dc_parser_t *abstract, const unsigned char *data,
 {
 	oceanic_vtpro_parser_t *parser = (oceanic_vtpro_parser_t *) abstract;
 
-	if (! parser_is_oceanic_vtpro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Reset the cache.
 	parser->cached = 0;
 	parser->divetime = 0;
@@ -211,9 +197,6 @@ oceanic_vtpro_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, uns
 static dc_status_t
 oceanic_vtpro_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata)
 {
-	if (! parser_is_oceanic_vtpro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	const unsigned char *data = abstract->data;
 	unsigned int size = abstract->size;
 
