@@ -28,6 +28,8 @@
 #include "parser-private.h"
 #include "array.h"
 
+#define ISINSTANCE(parser) dc_parser_isinstance((parser), &reefnet_sensus_parser_vtable)
+
 #define SAMPLE_DEPTH_ADJUST	13
 
 typedef struct reefnet_sensus_parser_t reefnet_sensus_parser_t;
@@ -52,7 +54,7 @@ static dc_status_t reefnet_sensus_parser_get_field (dc_parser_t *abstract, dc_fi
 static dc_status_t reefnet_sensus_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
 static dc_status_t reefnet_sensus_parser_destroy (dc_parser_t *abstract);
 
-static const parser_backend_t reefnet_sensus_parser_backend = {
+static const dc_parser_vtable_t reefnet_sensus_parser_vtable = {
 	DC_FAMILY_REEFNET_SENSUS,
 	reefnet_sensus_parser_set_data, /* set_data */
 	reefnet_sensus_parser_get_datetime, /* datetime */
@@ -62,16 +64,6 @@ static const parser_backend_t reefnet_sensus_parser_backend = {
 };
 
 
-static int
-parser_is_reefnet_sensus (dc_parser_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &reefnet_sensus_parser_backend;
-}
-
-
 dc_status_t
 reefnet_sensus_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int devtime, dc_ticks_t systime)
 {
@@ -86,7 +78,7 @@ reefnet_sensus_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, context, &reefnet_sensus_parser_backend);
+	parser_init (&parser->base, context, &reefnet_sensus_parser_vtable);
 
 	// Set the default values.
 	parser->atmospheric = ATM;
@@ -106,9 +98,6 @@ reefnet_sensus_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 static dc_status_t
 reefnet_sensus_parser_destroy (dc_parser_t *abstract)
 {
-	if (! parser_is_reefnet_sensus (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Free memory.	
 	free (abstract);
 
@@ -121,9 +110,6 @@ reefnet_sensus_parser_set_data (dc_parser_t *abstract, const unsigned char *data
 {
 	reefnet_sensus_parser_t *parser = (reefnet_sensus_parser_t*) abstract;
 
-	if (! parser_is_reefnet_sensus (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Reset the cache.
 	parser->cached = 0;
 	parser->divetime = 0;
@@ -138,7 +124,7 @@ reefnet_sensus_parser_set_calibration (dc_parser_t *abstract, double atmospheric
 {
 	reefnet_sensus_parser_t *parser = (reefnet_sensus_parser_t*) abstract;
 
-	if (! parser_is_reefnet_sensus (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	parser->atmospheric = atmospheric;
@@ -239,9 +225,6 @@ reefnet_sensus_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback
 {
 	reefnet_sensus_parser_t *parser = (reefnet_sensus_parser_t*) abstract;
 
-	if (! parser_is_reefnet_sensus (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	const unsigned char *data = abstract->data;
 	unsigned int size = abstract->size;
 
