@@ -29,6 +29,8 @@
 #include "parser-private.h"
 #include "array.h"
 
+#define ISINSTANCE(parser) dc_parser_isinstance((parser), &reefnet_sensuspro_parser_vtable)
+
 typedef struct reefnet_sensuspro_parser_t reefnet_sensuspro_parser_t;
 
 struct reefnet_sensuspro_parser_t {
@@ -51,7 +53,7 @@ static dc_status_t reefnet_sensuspro_parser_get_field (dc_parser_t *abstract, dc
 static dc_status_t reefnet_sensuspro_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
 static dc_status_t reefnet_sensuspro_parser_destroy (dc_parser_t *abstract);
 
-static const parser_backend_t reefnet_sensuspro_parser_backend = {
+static const dc_parser_vtable_t reefnet_sensuspro_parser_vtable = {
 	DC_FAMILY_REEFNET_SENSUSPRO,
 	reefnet_sensuspro_parser_set_data, /* set_data */
 	reefnet_sensuspro_parser_get_datetime, /* datetime */
@@ -61,16 +63,6 @@ static const parser_backend_t reefnet_sensuspro_parser_backend = {
 };
 
 
-static int
-parser_is_reefnet_sensuspro (dc_parser_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &reefnet_sensuspro_parser_backend;
-}
-
-
 dc_status_t
 reefnet_sensuspro_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int devtime, dc_ticks_t systime)
 {
@@ -85,7 +77,7 @@ reefnet_sensuspro_parser_create (dc_parser_t **out, dc_context_t *context, unsig
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, context, &reefnet_sensuspro_parser_backend);
+	parser_init (&parser->base, context, &reefnet_sensuspro_parser_vtable);
 
 	// Set the default values.
 	parser->atmospheric = ATM;
@@ -105,9 +97,6 @@ reefnet_sensuspro_parser_create (dc_parser_t **out, dc_context_t *context, unsig
 static dc_status_t
 reefnet_sensuspro_parser_destroy (dc_parser_t *abstract)
 {
-	if (! parser_is_reefnet_sensuspro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Free memory.	
 	free (abstract);
 
@@ -120,9 +109,6 @@ reefnet_sensuspro_parser_set_data (dc_parser_t *abstract, const unsigned char *d
 {
 	reefnet_sensuspro_parser_t *parser = (reefnet_sensuspro_parser_t*) abstract;
 
-	if (! parser_is_reefnet_sensuspro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Reset the cache.
 	parser->cached = 0;
 	parser->divetime = 0;
@@ -137,7 +123,7 @@ reefnet_sensuspro_parser_set_calibration (dc_parser_t *abstract, double atmosphe
 {
 	reefnet_sensuspro_parser_t *parser = (reefnet_sensuspro_parser_t*) abstract;
 
-	if (! parser_is_reefnet_sensuspro (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	parser->atmospheric = atmospheric;
@@ -228,9 +214,6 @@ reefnet_sensuspro_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callb
 {
 	reefnet_sensuspro_parser_t *parser = (reefnet_sensuspro_parser_t*) abstract;
 
-	if (! parser_is_reefnet_sensuspro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	const unsigned char header[4] = {0x00, 0x00, 0x00, 0x00};
 	const unsigned char footer[2] = {0xFF, 0xFF};
 
