@@ -27,6 +27,8 @@
 #include "context-private.h"
 #include "parser-private.h"
 
+#define ISINSTANCE(parser) dc_parser_isinstance((parser), &suunto_vyper_parser_vtable)
+
 typedef struct suunto_vyper_parser_t suunto_vyper_parser_t;
 
 struct suunto_vyper_parser_t {
@@ -43,7 +45,7 @@ static dc_status_t suunto_vyper_parser_get_field (dc_parser_t *abstract, dc_fiel
 static dc_status_t suunto_vyper_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
 static dc_status_t suunto_vyper_parser_destroy (dc_parser_t *abstract);
 
-static const parser_backend_t suunto_vyper_parser_backend = {
+static const dc_parser_vtable_t suunto_vyper_parser_vtable = {
 	DC_FAMILY_SUUNTO_VYPER,
 	suunto_vyper_parser_set_data, /* set_data */
 	suunto_vyper_parser_get_datetime, /* datetime */
@@ -53,16 +55,6 @@ static const parser_backend_t suunto_vyper_parser_backend = {
 };
 
 
-static int
-parser_is_suunto_vyper (dc_parser_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &suunto_vyper_parser_backend;
-}
-
-
 dc_status_t
 suunto_vyper_parser_create (dc_parser_t **out, dc_context_t *context)
 {
@@ -77,7 +69,7 @@ suunto_vyper_parser_create (dc_parser_t **out, dc_context_t *context)
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, context, &suunto_vyper_parser_backend);
+	parser_init (&parser->base, context, &suunto_vyper_parser_vtable);
 
 	// Set the default values.
 	parser->cached = 0;
@@ -93,9 +85,6 @@ suunto_vyper_parser_create (dc_parser_t **out, dc_context_t *context)
 static dc_status_t
 suunto_vyper_parser_destroy (dc_parser_t *abstract)
 {
-	if (! parser_is_suunto_vyper (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Free memory.	
 	free (abstract);
 
@@ -108,9 +97,6 @@ suunto_vyper_parser_set_data (dc_parser_t *abstract, const unsigned char *data,
 {
 	suunto_vyper_parser_t *parser = (suunto_vyper_parser_t *) abstract;
 
-	if (! parser_is_suunto_vyper (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Reset the cache.
 	parser->cached = 0;
 	parser->divetime = 0;
@@ -209,9 +195,6 @@ suunto_vyper_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsi
 static dc_status_t
 suunto_vyper_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata)
 {
-	if (! parser_is_suunto_vyper (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	const unsigned char *data = abstract->data;
 	unsigned int size = abstract->size;
 
