@@ -31,6 +31,8 @@
 #include "checksum.h"
 #include "array.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &uwatec_memomouse_device_vtable)
+
 #define EXITCODE(rc) \
 ( \
 	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
@@ -54,7 +56,7 @@ static dc_status_t uwatec_memomouse_device_dump (dc_device_t *abstract, dc_buffe
 static dc_status_t uwatec_memomouse_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t uwatec_memomouse_device_close (dc_device_t *abstract);
 
-static const device_backend_t uwatec_memomouse_device_backend = {
+static const dc_device_vtable_t uwatec_memomouse_device_vtable = {
 	DC_FAMILY_UWATEC_MEMOMOUSE,
 	uwatec_memomouse_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
@@ -64,15 +66,6 @@ static const device_backend_t uwatec_memomouse_device_backend = {
 	uwatec_memomouse_device_close /* close */
 };
 
-static int
-device_is_uwatec_memomouse (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &uwatec_memomouse_device_backend;
-}
-
 
 dc_status_t
 uwatec_memomouse_device_open (dc_device_t **out, dc_context_t *context, const char *name)
@@ -88,7 +81,7 @@ uwatec_memomouse_device_open (dc_device_t **out, dc_context_t *context, const ch
 	}
 
 	// Initialize the base class.
-	device_init (&device->base, context, &uwatec_memomouse_device_backend);
+	device_init (&device->base, context, &uwatec_memomouse_device_vtable);
 
 	// Set the default values.
 	device->port = NULL;
@@ -144,9 +137,6 @@ uwatec_memomouse_device_close (dc_device_t *abstract)
 {
 	uwatec_memomouse_device_t *device = (uwatec_memomouse_device_t*) abstract;
 
-	if (! device_is_uwatec_memomouse (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Close the device.
 	if (serial_close (device->port) == -1) {
 		free (device);
@@ -165,9 +155,6 @@ uwatec_memomouse_device_set_fingerprint (dc_device_t *abstract, const unsigned c
 {
 	uwatec_memomouse_device_t *device = (uwatec_memomouse_device_t*) abstract;
 
-	if (! device_is_uwatec_memomouse (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	if (size && size != 4)
 		return DC_STATUS_INVALIDARGS;
 
@@ -452,9 +439,6 @@ uwatec_memomouse_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	uwatec_memomouse_device_t *device = (uwatec_memomouse_device_t*) abstract;
 
-	if (! device_is_uwatec_memomouse (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Erase the current contents of the buffer.
 	if (!dc_buffer_clear (buffer)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
@@ -487,9 +471,6 @@ uwatec_memomouse_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 static dc_status_t
 uwatec_memomouse_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
-	if (! device_is_uwatec_memomouse (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	dc_buffer_t *buffer = dc_buffer_new (0);
 	if (buffer == NULL)
 		return DC_STATUS_NOMEMORY;
@@ -512,7 +493,7 @@ uwatec_memomouse_device_foreach (dc_device_t *abstract, dc_dive_callback_t callb
 dc_status_t
 uwatec_memomouse_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
-	if (abstract && !device_is_uwatec_memomouse (abstract))
+	if (abstract && !ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	// Parse the data stream to find the total number of dives.
