@@ -29,6 +29,8 @@
 #include "irda.h"
 #include "array.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &uwatec_smart_device_vtable)
+
 #define EXITCODE(rc) \
 ( \
 	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
@@ -48,7 +50,7 @@ static dc_status_t uwatec_smart_device_dump (dc_device_t *abstract, dc_buffer_t
 static dc_status_t uwatec_smart_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t uwatec_smart_device_close (dc_device_t *abstract);
 
-static const device_backend_t uwatec_smart_device_backend = {
+static const dc_device_vtable_t uwatec_smart_device_vtable = {
 	DC_FAMILY_UWATEC_SMART,
 	uwatec_smart_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
@@ -58,15 +60,6 @@ static const device_backend_t uwatec_smart_device_backend = {
 	uwatec_smart_device_close /* close */
 };
 
-static int
-device_is_uwatec_smart (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &uwatec_smart_device_backend;
-}
-
 
 static void
 uwatec_smart_discovery (unsigned int address, const char *name, unsigned int charset, unsigned int hints, void *userdata)
@@ -163,7 +156,7 @@ uwatec_smart_device_open (dc_device_t **out, dc_context_t *context)
 	}
 
 	// Initialize the base class.
-	device_init (&device->base, context, &uwatec_smart_device_backend);
+	device_init (&device->base, context, &uwatec_smart_device_vtable);
 
 	// Set the default values.
 	device->socket = NULL;
@@ -219,9 +212,6 @@ uwatec_smart_device_close (dc_device_t *abstract)
 {
 	uwatec_smart_device_t *device = (uwatec_smart_device_t*) abstract;
 
-	if (! device_is_uwatec_smart (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Close the device.
 	if (irda_socket_close (device->socket) == -1) {
 		free (device);
@@ -240,9 +230,6 @@ uwatec_smart_device_set_fingerprint (dc_device_t *abstract, const unsigned char
 {
 	uwatec_smart_device_t *device = (uwatec_smart_device_t*) abstract;
 
-	if (! device_is_uwatec_smart (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	if (size && size != 4)
 		return DC_STATUS_INVALIDARGS;
 
@@ -261,9 +248,6 @@ uwatec_smart_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	uwatec_smart_device_t *device = (uwatec_smart_device_t*) abstract;
 	dc_status_t rc = DC_STATUS_SUCCESS;
 
-	if (! device_is_uwatec_smart (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Erase the current contents of the buffer.
 	if (!dc_buffer_clear (buffer)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
@@ -403,9 +387,6 @@ uwatec_smart_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 static dc_status_t
 uwatec_smart_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
-	if (! device_is_uwatec_smart (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	dc_buffer_t *buffer = dc_buffer_new (0);
 	if (buffer == NULL)
 		return DC_STATUS_NOMEMORY;
@@ -428,7 +409,7 @@ uwatec_smart_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback,
 dc_status_t
 uwatec_smart_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
-	if (abstract && !device_is_uwatec_smart (abstract))
+	if (abstract && !ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	const unsigned char header[4] = {0xa5, 0xa5, 0x5a, 0x5a};
